@@ -66,6 +66,18 @@ fi
 
 [ -z "${START_REDIS}" ] && START_REDIS=true
 [ -z "${REDIS_PORT}" ] && REDIS_PORT=40001
+if [[ ${UPGRADE} == true ]]; then
+  # Detect the present of shelf controller in DCP-R or DCP-2 cluster
+  if [[ -f "/proc/device-tree/dcpr" ]] || [[ -f "/proc/device-tree/dcp2" ]]; then
+    ip_controller=$(redis-cli -p 50002 get "${REDIS_DATA_HASH_TAG}":so:mounts | jq -r '.[] | to_entries | .[] | select(.value.instance_type=="CONF" and .value.node_type=="Controller") | .value.ip')
+    if [[ ! -z "${ip_controller}" ]]; then
+      LOG "$(basename $0) Found controller at ip: ${ip_controller}"
+      # Use redis cluster port to be able to redirect to correct master, controller.
+      REDIS_PORT=50001
+    fi
+  fi
+fi
+
 CLI_BIN="/usr/bin/redis-cli"
 
 LOG "Triggered: ${base_path} START_REDIS=${START_REDIS} REDIS_PORT=${REDIS_PORT}"
@@ -111,6 +123,30 @@ function wait_redis_persistent() {
   done
 }
 
+# Usage: cidr_to_netmask <number within 1-32>
+# Return the netmask for a given network and CIDR.
+cidr_to_netmask() {
+    [ -z "$1" ] && return 1
+    value=$(( 0xffffffff ^ ((1 << (32 - $1)) - 1) ))
+    echo -n "$(( (value >> 24) & 0xff )).$(( (value >> 16) & 0xff ))."
+    echo -n "$(( (value >> 8) & 0xff )).$(( value & 0xff ))"
+    echo
+}
+
+get_ip_conf() {
+  if [[ $(ip -V) == "ip utility"* ]]; then
+    ipaddr=$(ip -4 -j -br addr show dev $1 | jq -r .[0].addr_info[0].local)
+    prefixlen=$(ip -4 -j -br addr show dev $1 | jq -r .[0].addr_info[0].prefixlen)
+    netmask=$(cidr_to_netmask $prefixlen)
+    gateway=$(ip -4 -j route show dev $1 default | jq -r '.[0].gateway // empty')
+  else
+    ifconfig=$(ifconfig $1 | grep "inet addr" | xargs)
+    ipaddr=$(echo ${ifconfig} | sed 's/.*addr:\(\S*\).*/\1/')
+    netmask=$(echo ${ifconfig} | sed 's/.*Mask:\(\S*\).*/\1/')
+    gateway=$(ip route show $1 | grep default | sed 's/.*via\s\(\S*\).*/\1/')
+  fi
+}
+
 if [[ ${START_REDIS} == true ]]; then
   WORKSPACE="${PERSISTENT_DIRECTORY}/etc/sw_storage/redis/"
   if [[ ! -d "$PERSISTENT_DIRECTORY" ]]; then
@@ -136,39 +172,32 @@ if [[ ${START_REDIS} == true ]]; then
   fi
 fi
 
-mgmt_ip=""
-mgmt_netmask=""
-mgmt_gateway=""
-if [[ -f /proc/device-tree/dcpr ]]; then
-  ifconfig=$(ifconfig eth0.2 | grep "inet addr" | xargs)
-  mgmt_ip=$(echo ${ifconfig} | sed 's/.*addr:\(\S*\).*/\1/')
-  mgmt_netmask=$(echo ${ifconfig} | sed 's/.*Mask:\(\S*\).*/\1/')
-  mgmt_gateway=$(ip route show eth0.2 | grep default | sed 's/.*via\s\(\S*\).*/\1/')
-  LOG "Collected eth0.2 network settings: [${mgmt_ip}] [${mgmt_netmask}] [${mgmt_gateway}]"
+ipaddr=""
+netmask=""
+gateway=""
+if [[ -f "/proc/device-tree/dcpr" ]] || [[ -f "/proc/device-tree/dcpm2" ]]; then
+  # Maintain the compatibility with releases older than R11
+  get_ip_conf eth0.2
+  LOG "Collected eth0.2 network settings: [${ipaddr}] [${netmask}] [${gateway}]"
 elif [[ -f "/proc/device-tree/is_chassis" ]]; then
-  br0_ifconfig=$(ifconfig br0 | grep "inet addr" | xargs)
-  mgmt_ip=$(echo ${br0_ifconfig} | sed 's/.*addr:\(\S*\).*/\1/')
-  mgmt_netmask=$(echo ${br0_ifconfig} | sed 's/.*Mask:\(\S*\).*/\1/')
-  mgmt_gateway=$(ip route show br0 | grep default | sed 's/.*via\s\(\S*\).*/\1/')
-  LOG "Collected br0 network settings: [${mgmt_ip}] [${mgmt_netmask}] [${mgmt_gateway}]"
+  get_ip_conf br0
+  LOG "Collected br0 network settings: [${ipaddr}] [${netmask}] [${gateway}]"
 fi
 
 # Both DCP-R and DCP-2/DCP-M are using same redis conf path for mgmt-ip config: config/br0/ipv4Settings, is that a bug?
-mgmt_config="{\"value\":{\"ipAddress\":\"${mgmt_ip}\",\"netmask\":\"${mgmt_netmask}\",\"gateway\":\"${mgmt_gateway}\"}}"
+mgmt_config="{\"value\":{\"ipAddress\":\"${ipaddr}\",\"netmask\":\"${netmask}\",\"gateway\":\"${gateway}\"}}"
 LOG "Creating config Prefix: dh/network Path: config/br0/ipv4Settings Value: ${mgmt_config}"
-${CLI_BIN} -p ${REDIS_PORT} DH.CREATE_CONFIG_KEY ${REDIS_CONF_HASH_TAG} dh/network config/br0/ipv4Settings ${mgmt_config}
-${CLI_BIN} -p ${REDIS_PORT} DH.SET               ${REDIS_CONF_HASH_TAG}:dh/network:config/br0/ipv4Settings ${mgmt_config}
+${CLI_BIN} -c -p ${REDIS_PORT} EVAL   'redis.call("DH.CREATE_CONFIG_KEY", KEYS[1], ARGV[1], ARGV[2], ARGV[3])' 1 ${REDIS_CONF_HASH_TAG} dh/network config/br0/ipv4Settings ${mgmt_config}
+${CLI_BIN} -c -p ${REDIS_PORT} DH.SET ${REDIS_CONF_HASH_TAG}:dh/network:config/br0/ipv4Settings ${mgmt_config}
 
 # Both DCP-R and DCP-2/DCP-M use same local eth0 nw interface
 # eth0 does not have any gateway configured
-eth0_ifconfig=$(ifconfig eth0 | grep "inet addr" | xargs)
-eth0_ipaddr=$(echo ${eth0_ifconfig} | sed 's/.*addr:\(\S*\).*/\1/')
-eth0_netmask=$(echo ${eth0_ifconfig} | sed 's/.*Mask:\(\S*\).*/\1/')
-LOG "Collected eth0 network settings: [${eth0_ipaddr}] [${eth0_netmask}] "
-eth0_config="{\"value\":{\"ipAddress\":\"${eth0_ipaddr}\",\"netmask\":\"${eth0_netmask}\",\"gateway\":\"\"}}"
+get_ip_conf eth0
+LOG "Collected eth0 network settings: [${ipaddr}] [${netmask}] "
+eth0_config="{\"value\":{\"ipAddress\":\"${ipaddr}\",\"netmask\":\"${netmask}\",\"gateway\":\"\"}}"
 LOG "Creating config Prefix: dh/network Path: config/eth0/ipv4Settings Value: ${eth0_config}"
-${CLI_BIN} -p ${REDIS_PORT} DH.CREATE_CONFIG_KEY ${REDIS_CONF_HASH_TAG} dh/network config/eth0/ipv4Settings ${eth0_config}
-${CLI_BIN} -p ${REDIS_PORT} DH.SET               ${REDIS_CONF_HASH_TAG}:dh/network:config/eth0/ipv4Settings ${eth0_config}
+${CLI_BIN} -c -p ${REDIS_PORT} EVAL   'redis.call("DH.CREATE_CONFIG_KEY", KEYS[1], ARGV[1], ARGV[2], ARGV[3])' 1 ${REDIS_CONF_HASH_TAG} dh/network config/eth0/ipv4Settings ${eth0_config}
+${CLI_BIN} -c -p ${REDIS_PORT} DH.SET ${REDIS_CONF_HASH_TAG}:dh/network:config/eth0/ipv4Settings ${eth0_config}
 
 # Save DNS settings
 cat /etc/resolv.conf | {
@@ -179,8 +208,8 @@ cat /etc/resolv.conf | {
   LOG "Collected DNS settings: [${dns1}] [${dns2}]"
   dns_config="{\"value\":{\"primaryDnsIpAddress\":\"${dns1}\",\"secondaryDnsIpAddress\":\"${dns2}\"}}"
   LOG "Creating config Prefix: dh/network Path: config/dnsSettings Value: ${dns_config}"
-  ${CLI_BIN} -p ${REDIS_PORT} DH.CREATE_CONFIG_KEY ${REDIS_CONF_HASH_TAG} dh/network config/dnsSettings "${dns_config}"
-  ${CLI_BIN} -p ${REDIS_PORT} DH.SET               ${REDIS_CONF_HASH_TAG}:dh/network:config/dnsSettings "${dns_config}"
+  ${CLI_BIN} -c -p ${REDIS_PORT} EVAL   'redis.call("DH.CREATE_CONFIG_KEY", KEYS[1], ARGV[1], ARGV[2], ARGV[3])' 1 ${REDIS_CONF_HASH_TAG} dh/network config/dnsSettings "${dns_config}"
+  ${CLI_BIN} -c -p ${REDIS_PORT} DH.SET ${REDIS_CONF_HASH_TAG}:dh/network:config/dnsSettings "${dns_config}"
 }
 
 if [[ ${START_REDIS} == true ]]; then
