@@ -731,6 +731,17 @@ ${CLI_BIN} -p ${REDIS_PORT} DH.CREATE_CONFIG_KEY ${REDIS_CONF_HASH_TAG} dh/netwo
 timestamp=`/bin/date`
 ${CLI_BIN} -p ${REDIS_PORT} set ${REDIS_CONF_HASH_TAG}:so:upgrade/timestamp "{\"value\":\"${timestamp}\"}"
 
+# Handle shadow file for R6
+user_hashes='{}'
+for user_name in admin; do
+  LOG "Migrating shadow hash to redis for user: $user_name"
+  user_hash=$(getent shadow $user_name | tr -d '\n')
+  nested_object=$(${JQ_BIN} -n --arg key "$user_name" --arg hash "$user_hash" '{($key): {"hash": $hash}}')
+  user_hashes=$(echo "$user_hashes" "$nested_object" | ${JQ_BIN} -s '.[0] * .[1]')
+done
+json_flat=$(echo "$user_hashes" | ${JQ_BIN} -c -r)
+${CLI_BIN} -p ${REDIS_PORT} DH.CREATE_CONFIG_KEY ${REDIS_CONF_HASH_TAG} dh/system/settings config/cliUsers "{\"value\":$json_flat}"
+
 ${CLI_BIN} -p ${REDIS_PORT} BGREWRITEAOF
 
 timer=30
