@@ -48,7 +48,7 @@ then
 fi
 
 #Upgrade firmware
-INSTALLING_FIRMWARE_VERSION=0.4.32
+INSTALLING_FIRMWARE_VERSION=0.4.39
 INSTALLING_FIRMWARE_FILE="${FIRMWARE_DIRECTORY}/vsc7512-${INSTALLING_FIRMWARE_VERSION}.tar"
 VSC_DH_FILE="/tmp/device_handlers/vsc751x"
 RUNNING_FIRMWARE_VERSION=""
@@ -63,12 +63,12 @@ then
   [[ -f /usr/sbin/smartmanager/smartmanager.js ]] && FW_VER_PATH="data/runningFirmwareVersion"
   RUNNING_FIRMWARE_VERSION=$(tiny_dh_client -d "$VSC_DH_FILE" -c gets -a ${FW_VER_PATH})
 
-  # Don't upgrade DCP-F to 0.4.32
-  if [[ -f /proc/device-tree/dcpf ]]
+  # Don't upgrade DCP-F A22, R22, DE22 from 0.4.30 (Bypass this check for VG which has sfp_tx_force_enable-gpios pin)
+  if [[ -f /proc/device-tree/dcpf && ! -f /proc/device-tree/sfp0/sfp_tx_force_enable-gpios ]]
   then
     RUNNING_FIRMWARE_REVVERSION="${RUNNING_FIRMWARE_VERSION##0.4.}"
     INSTALLING_FIRMWARE_REVVERSION="${INSTALLING_FIRMWARE_VERSION##0.4.}"
-    if (( RUNNING_FIRMWARE_REVVERSION == 30 && INSTALLING_FIRMWARE_REVVERSION == 32 ))
+    if (( RUNNING_FIRMWARE_REVVERSION >= 30 ))
     then
       LOG "Will not upgrade vsc firmware from $RUNNING_FIRMWARE_VERSION to $INSTALLING_FIRMWARE_VERSION on DCP-F."
       exit 0
